/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package com.qiniu.android.dns.dns

import com.qiniu.android.dns.Record
import java.io.ByteArrayOutputStream
import java.io.DataOutputStream
import java.io.IOException
import java.net.IDN

internal class DnsRequest(messageId: Short, opCode: Int, rd: Int, recordType: Int, host: String?) :
    DnsMessage() {
    @JvmField val recordType: Int
    private val host: String?

    constructor(
        messageId: Short,
        recordType: Int,
        host: String?
    ) : this(messageId, 0, 1, recordType, host)

    init {
        this.messageId = messageId
        this.opCode = opCode
        rD = rd
        this.recordType = recordType
        this.host = host
    }

    @Throws(IOException::class)
    fun toDnsQuestionData(): ByteArray {
        if (host.isNullOrEmpty()) {
            throw IOException("host can not empty")
        }
        if (opCode != OpCodeQuery &&
            opCode != OpCodeIQuery &&
            opCode != OpCodeStatus &&
            opCode != OpCodeUpdate) {
            throw IOException("opCode is not valid")
        }
        if (rD != 0 && rD != 1) {
            throw IOException("rd is not valid")
        }
        if (recordType != Record.TYPE_A &&
            recordType != Record.TYPE_AAAA &&
            recordType != Record.TYPE_CNAME &&
            recordType != Record.TYPE_TXT) {
            throw IOException("recordType is not valid")
        }
        val baos = ByteArrayOutputStream(512)
        val dos = DataOutputStream(baos)
        // 16 bit id
        dos.writeShort(messageId.toInt())
        // |00|01|02|03|04|05|06|07|
        // |QR|  OPCODE   |AA|TC|RD|
        dos.writeByte((opCode shl 3) + rD)
        // |00|01|02|03|04|05|06|07|
        // |RA|r1|r2|r3| RCODE     |
        dos.writeByte(0x00)
        dos.writeByte(0x00)
        dos.writeByte(0x01) // QDCOUNT (number of entries in the question section)
        dos.writeByte(0x00)
        dos.writeByte(0x00) // ANCOUNT
        dos.writeByte(0x00)
        dos.writeByte(0x00) // NSCOUNT
        dos.writeByte(0x00)
        dos.writeByte(0x00) // ARCOUNT
        for (s in
            host
                .split("[.\u3002\uFF0E\uFF61]".toRegex())
                .dropLastWhile { it.isEmpty() }
                .toTypedArray()) {
            if (s.length > 63) {
                throw IOException("host part is too long")
            }
            val buffer = IDN.toASCII(s).toByteArray()
            dos.write(buffer.size)
            dos.write(buffer, 0, buffer.size) //
        }
        dos.writeByte(0x00) /* terminating zero */
        dos.writeByte(0x00)
        dos.writeByte(recordType)
        dos.writeByte(0x00)
        dos.writeByte(0x01) /* IN - "the Internet" */
        return baos.toByteArray()
    }
}
