/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package com.qiniu.android.dns.local

import com.qiniu.android.dns.Domain
import com.qiniu.android.dns.IResolver
import com.qiniu.android.dns.NetworkInfo
import com.qiniu.android.dns.Record
import com.qiniu.android.dns.dns.DnsUdpResolver
import java.io.IOException

/** Created by bailong on 15/7/10. */
class HijackingDetectWrapper(private val resolver: DnsUdpResolver) : IResolver {

    @Throws(IOException::class)
    override fun resolve(domain: Domain?, info: NetworkInfo?): Array<Record?>? {
        val records = resolver.resolve(domain, info)
        domain?.let {
            if (domain.hasCname) {
                var cname = false
                var server: String? = null
                records?.let {
                    for (r in records) {
                        if (r?.isCname == true) {
                            cname = true
                            server = r.server
                            break
                        }
                    }
                }
                if (!cname) {
                    throw DnshijackingException(domain.domain, server)
                }
            }
            if (domain.maxTtl != 0) {
                records?.let {
                    for (r in records) {
                        if (r?.isCname == false) {
                            if (r.ttl > domain.maxTtl) {
                                throw DnshijackingException(domain.domain, r.server, r.ttl)
                            }
                        }
                    }
                }
            }
        }
        return records
    }
}
