/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package com.qiniu.android.dns.local

import com.qiniu.android.dns.Domain
import com.qiniu.android.dns.NetworkInfo
import com.qiniu.android.dns.Record
import java.util.LinkedList
import java.util.concurrent.ConcurrentHashMap

/** Created by bailong on 15/6/18. */
class Hosts {
    private val hosts: MutableMap<String, LinkedList<Value>> = ConcurrentHashMap()

    @Synchronized
    fun query(domain: Domain, info: NetworkInfo): Array<Record?>? {
        var values = hosts[domain.domain]
        if (values.isNullOrEmpty()) {
            return null
        }
        if (values.size > 1) {
            val first = values[0]
            values.removeAt(0)
            values.add(first)
        }
        values = filter(values, info)
        return toRecords(values)
    }

    private fun filter(origin: LinkedList<Value>?, info: NetworkInfo): LinkedList<Value>? {
        if (origin == null) {
            return null
        }
        val normal = LinkedList<Value>()
        val special = LinkedList<Value>()
        for (v in origin) {
            if (v.provider == NetworkInfo.ISP_GENERAL) {
                normal.add(v)
            } else if (v.provider == info.provider) {
                special.add(v)
            }
        }
        return if (special.size != 0) {
            special
        } else normal
    }

    private fun toRecords(vals: LinkedList<Value>?): Array<Record?>? {
        if (vals == null) {
            return null
        }
        val size = vals.size
        val records: MutableList<Record?> = ArrayList()
        for (i in 0 until size) {
            val v = vals[i]
            if (v.record?.value != null) {
                records.add(v.record)
            }
        }
        return records.toTypedArray<Record?>()
    }

    @Synchronized
    fun put(domain: String, `val`: Value): Hosts {
        var vals = hosts[domain]
        if (vals == null) {
            vals = LinkedList()
        }
        vals.add(`val`)
        hosts[domain] = vals
        return this
    }

    fun put(domain: String, record: Record?): Hosts {
        put(domain, Value(record))
        return this
    }

    class Value
    @JvmOverloads
    constructor(val record: Record?, val provider: Int = NetworkInfo.ISP_GENERAL) {
        override fun equals(o: Any?): Boolean {
            if (this === o) {
                return true
            }
            if (o !is Value) {
                return false
            }
            if (record === o.record) {
                return true
            }
            return if (record == null || o.record == null) {
                false
            } else record.value == o.record.value && provider == o.provider
        }
    }
}
